use crate::Error;
use zip::{ZipWriter, ZipArchive as ZipReader, write::SimpleFileOptions, CompressionMethod};
use std::fs::{self, read_dir, File};
use std::path::PathBuf;

fn pocket_options() -> SimpleFileOptions {
    SimpleFileOptions::default()
        .compression_method(CompressionMethod::Bzip2)
        .compression_level(Some(6))
        .unix_permissions(0o755)
}

pub struct Pocket {
    output: Option<ZipWriter<File>>,
    input: Option<ZipReader<File>>,
}

impl Pocket {
    pub fn try_read(path: &PathBuf) -> Result<Self, Error> {
        let f = File::open(path)?;
        let z = ZipReader::new(f)?;
        Ok(Pocket{ output: None, input: Some(z) })
    }
    pub fn try_write(path: &PathBuf) -> Result<Self, Error> {
        let f = File::create(path)?;
        let z = ZipWriter::new(f);
        Ok(Pocket { output: Some(z), input: None })
    }
    pub fn is_read(&self) -> bool {
        self.input.is_some() && self.output.is_none()
    }
    pub fn is_write(&self) -> bool {
        self.output.is_some() && self.input.is_none()
    }
    pub fn insert(&mut self, path: &PathBuf, consume: bool) -> Result<(), Error> {
        if self.is_read() {
            return Err(Error::ReadMode);
        }
        let mut zw = self.output.as_mut().unwrap();
        zw.start_file_from_path(path, pocket_options())?;
        let mut r = File::open(path)?;
        std::io::copy(&mut r, &mut zw)?;
        if consume {
            std::fs::remove_file(path)?;
        }
        Ok(())
    }
    pub fn insert_dir(&mut self, path: &PathBuf, consume: bool) -> Result<(), Error> {
        if self.is_read() {
            return Err(Error::ReadMode);
        }
        let zw = self.output.as_mut().unwrap();
        zw.add_directory_from_path(path, pocket_options())?;
        for entry in read_dir(path)? {
            let entry = entry?;
            let p = entry.path();
            if p.is_dir() {
                self.insert_dir(&p, consume)?
            } else if p.is_file() {
                self.insert(&p, false)?;
            }
        };
        if consume {
            fs::remove_dir_all(path)?;
        }
        Ok(())
    }
}
